
create sequence ID_SEQUENCE_MESSAGE;
create sequence ID_SEQUENCE_CONVERSATION;
create sequence ID_SEQUENCE_FILE_REF;

create table CONVERSATION
(
    ID                         number(18) not null,
    CONVERSATION_ID            varchar(500) not null,
    SEQUENCE_OUT_BLOCKED       number(1) not null,
    SEQUENCE_OUT_OFFSET        number(18) not null,
    SEQUENCE_OFFSET            number(18) not null,
    constraint PK_CONVERSATION primary key (ID),
    constraint UQ_CONVERSATION unique (CONVERSATION_ID)
);

create table MESSAGE
(
    ID                      number(18) not null,
    MESSAGE_ID              varchar(500) not null,
    FOLDER                  varchar(255) null,
    CREATION_TIME           number(18) null,
    INBOUND                 number(1) not null,
    CONVERSATION            number(18) null,
    SEQUENCE_NUMBER         number(18) null,
    MESSAGE_TYPE            varchar(100) null,
    SCHEMA_VERSION          varchar(50) null,
    SCHEMA_SET              varchar(50) null,
    PROTOCOL                varchar(10) null,
    RECEIVER_ID             varchar(30) null,
    SENDER_ID               varchar(30) null,
    MESSAGE_TIME            number(18) null,
    REFERENCE_ID            number(18) null,
    TTL                     number(18) null,
    PACKAGING_ID            varchar(50) null,
    SIGNED_ACK_REQUESTED    number(1) null,
    LOGINFO                 varchar(128) null,
    TESTFLAG                number(1) null,
    STATUS                  number(2) null,
    ACK_RECEIVED            number(1) null,
    ADAPTER_ID              varchar(50) null,
    MESSENGER_ID            number(3) null,
    constraint PK_MESSAGES primary key (ID),
    constraint UQ_MESSAGE unique (MESSAGE_ID, INBOUND),
    constraint FK_MESSAGE_CONVERSATION foreign key (CONVERSATION) references CONVERSATION(ID),
    constraint FK_MESSAGE_MESSAGE foreign key (REFERENCE_ID) references MESSAGE(ID)
);

create index IDX_MESSAGE_CONVERSATION ON MESSAGE (CONVERSATION);
create index IDX_MESSAGE_REFID ON MESSAGE (REFERENCE_ID);
create index IDX_MESSAGE_MESSAGE_TIME ON MESSAGE (MESSAGE_TIME);

create table FILE_REF
(
    ID                      number(18) not null,
    MESSAGE_ID              number(18) not null,
    FILENAME                varchar(255) not null,
    constraint PK_FILE_REFS primary key (ID),
    constraint UQ_FILE_REF unique (MESSAGE_ID, FILENAME),
    constraint FK_FILE_REFS_MESSAGE foreign key (MESSAGE_ID) references MESSAGE(ID)
);

create index IDX_FILE_REF_MSGID ON FILE_REF (MESSAGE_ID);

create table LOG_MESSAGE
(
    ID                      int not null,
    TYPE                    int not null,
    BEGIN_COMPONENT         int not null,
    END_COMPONENT           int not null,
    TEXT                    varchar(255) not null,
    constraint PK_LOG_MESSAGE primary key(ID)
);

create table MESSAGE_LOG
(
    ID                      number(18) not null,
    TIMESTAMP               number(18) not null,
    MESSAGE_ID              number(18) not null,
    TEXT_ID                 int null,
    ADDITIONAL_TEXT         varchar(512) null,
    constraint PK_MESSAGE_LOG primary key (ID),
    constraint FK_MESSAGE_LOG_MESSAGE foreign key (MESSAGE_ID) references MESSAGE(ID),
    constraint FK_MESSAGE_LOG_LOG_MESSAGE foreign key (TEXT_ID) references LOG_MESSAGE(ID)
);

create index IDX_MSGLOG_MSGID ON MESSAGE_LOG (MESSAGE_ID);
create index IDX_MSGLOG_TIMESTAMP ON MESSAGE_LOG (TIMESTAMP);

create table INBOUND_QUEUE
(
    MESSAGE_ID              number(18) not null,
    TIMESECONDS             number(18) null,
    ADAPTER_ID              varchar(50) null,
    SEQUENCE_NUMBER         number(18) null,
    NEXT_TRANSMISSION       number(18) null,
    MESSENGER_ID            number(3) null,
    ACK_SENT                number(3) null,
    constraint PK_INBOUND_QUEUE primary key (MESSAGE_ID),
    constraint FK_INBOUND_QUEUE_MESSAGE foreign key (MESSAGE_ID) references MESSAGE(ID)
);

create table OUTBOUND_QUEUE
(
    MESSAGE_ID              number(18) not null,
    TIMESECONDS             number(18) null,
    RECEIVER_URL            varchar(512) null,
    PACKAGING               varchar(20) null,
    TIME_TO_LIVE            number(18) null,
    RETRY_INTERVAL          number(10) null,
    RETRIES                 number(3) null,
    PARALLEL_DELIVERIES     number(10) null,
    ACK_REQUIRED            number(1) null,
    SENDER_ID               varchar(30) null,
    RECEIVER_ID             varchar(30) null,
    LAST_TRANSMISSION       number(18) null,
    MESSENGER_ID            number(3) null,
    constraint PK_OUTBOUND_QUEUE primary key (MESSAGE_ID),
    constraint FK_OUTBOUND_QUEUE_MESSAGE foreign key (MESSAGE_ID) references MESSAGE(ID)
);

create table MDNDATA
(
    MESSAGE_ID              number(18) not null,
    DIGEST                  varchar(512) null,
    DIGEST_ALGORITHM        varchar(30) null,
    constraint PK_MDNDATA primary key (MESSAGE_ID),
    constraint FK_MDNDATA_MESSAGE foreign key (MESSAGE_ID) references MESSAGE(ID)
);

create table ADAPTER_INFO
(
    ADAPTER_ID              varchar(50) not null,
    MESSENGER_ID            number(3) not null,
    ADDRESS                 varchar(128) null,
    MAXIMUM_THREADS         number(6) null,
    PROCESSING_TIMEOUT      number(10) null,
    FEATURES                varchar(255) null,
    constraint PK_ADAPTER_INFO primary key(ADAPTER_ID, MESSENGER_ID)
);
